/**
 * Copyright (c) 2007-2013, 2021 Borland Software Corporation, CEA LIST, Artal and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 * Alexander Shatalin (Borland) - initial API and implementation
 * Artem Tikhomirov (Borland) - [257632] do not rely on EditPart presence for element deletion
 * Michael Golubev (Montages) - #386838 - migrate to Xtend2
 * Etienne Allogo (ARTAL) - etienne.allogo@artal.fr - Bug 569174 : 1.4 Merge papyrus extension templates into codegen.xtend
 */
package xpt.diagram.editpolicies;

import com.google.common.collect.Iterables;
import com.google.inject.Inject;
import com.google.inject.Singleton;
import java.util.List;
import org.eclipse.emf.common.util.EList;
import org.eclipse.papyrus.gmf.codegen.gmfgen.FeatureLinkModelFacet;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenChildNode;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenCompartment;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenLink;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenNode;
import org.eclipse.papyrus.gmf.codegen.gmfgen.LinkModelFacet;
import org.eclipse.papyrus.gmf.codegen.gmfgen.TypeLinkModelFacet;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import utils.UtilsItemSemanticEditPolicy;
import xpt.Common;
import xpt.editor.VisualIDRegistry;

/**
 * This template should be called only for non-design nodes (modelFacet != null)
 * 	because *ItemSemanticEditPolicy responsible for dealing with semantic model
 * 	elements and meaningless (should not be generated) for pure design nodes.
 */
@Singleton
@SuppressWarnings("all")
public class NodeItemSemanticEditPolicy {
  @Inject
  @Extension
  private Common _common;

  @Inject
  @Extension
  private Utils_qvto _utils_qvto;

  @Inject
  @Extension
  private UtilsItemSemanticEditPolicy _utilsItemSemanticEditPolicy;

  @Inject
  private BaseItemSemanticEditPolicy xptBaseItemSemanticEditPolicy;

  @Inject
  private childContainerCreateCommand xptChildContainerCreateCommand;

  @Inject
  private linkCommands xptLinkCommands;

  @Inject
  private VisualIDRegistry xptVisualIDRegistry;

  public CharSequence className(final GenNode it) {
    StringConcatenation _builder = new StringConcatenation();
    String _itemSemanticEditPolicyClassName = it.getItemSemanticEditPolicyClassName();
    _builder.append(_itemSemanticEditPolicyClassName);
    return _builder;
  }

  public CharSequence packageName(final GenNode it) {
    StringConcatenation _builder = new StringConcatenation();
    String _editPoliciesPackageName = it.getDiagram().getEditPoliciesPackageName();
    _builder.append(_editPoliciesPackageName);
    return _builder;
  }

  public CharSequence qualifiedClassName(final GenNode it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _packageName = this.packageName(it);
    _builder.append(_packageName);
    _builder.append(".");
    CharSequence _className = this.className(it);
    _builder.append(_className);
    return _builder;
  }

  public CharSequence fullPath(final GenNode it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _qualifiedClassName = this.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    return _builder;
  }

  public CharSequence NodeItemSemanticEditPolicy(final GenNode it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _copyright = this._common.copyright(it.getDiagram().getEditorGen());
    _builder.append(_copyright);
    _builder.newLineIfNotEmpty();
    _builder.append("package ");
    CharSequence _packageName = this.packageName(it);
    _builder.append(_packageName);
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _generatedClassComment = this._common.generatedClassComment();
    _builder.append(_generatedClassComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    CharSequence _className = this.className(it);
    _builder.append(_className);
    _builder.append(" extends ");
    CharSequence _qualifiedClassName = this.xptBaseItemSemanticEditPolicy.qualifiedClassName(it.getDiagram());
    _builder.append(_qualifiedClassName);
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _defaultConstructor = this.xptBaseItemSemanticEditPolicy.defaultConstructor(it);
    _builder.append(_defaultConstructor, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _childContainerCreateCommand = this.xptChildContainerCreateCommand.childContainerCreateCommand(it.getChildNodes());
    _builder.append(_childContainerCreateCommand, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.newLine();
    {
      boolean _isUsingDeleteService = it.isUsingDeleteService();
      if (_isUsingDeleteService) {
        _builder.append("\t");
        CharSequence _generatedMemberComment = this._common.generatedMemberComment();
        _builder.append(_generatedMemberComment, "\t");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        CharSequence _destroyElementCommandByService = this._utilsItemSemanticEditPolicy.getDestroyElementCommandByService(it);
        _builder.append(_destroyElementCommandByService, "\t");
        _builder.newLineIfNotEmpty();
      } else {
        _builder.append("\t");
        CharSequence _destroyElementCommand = this.getDestroyElementCommand(it);
        _builder.append(_destroyElementCommand, "\t");
        _builder.newLineIfNotEmpty();
        {
          boolean _hasChildrenOrCompartments = this._utils_qvto.hasChildrenOrCompartments(it);
          if (_hasChildrenOrCompartments) {
            _builder.append("\t");
            CharSequence _addDestroyChildNodesCommand = this.addDestroyChildNodesCommand(it);
            _builder.append(_addDestroyChildNodesCommand, "\t");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    _builder.newLine();
    _builder.append("\t");
    CharSequence _linkCommands = this.xptLinkCommands.linkCommands(it);
    _builder.append(_linkCommands, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }

  public CharSequence getDestroyElementCommand(final GenNode it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("protected org.eclipse.gef.commands.Command getDestroyElementCommand(org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest req) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.gmf.runtime.notation.View view = (org.eclipse.gmf.runtime.notation.View) getHost().getModel();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.gmf.runtime.emf.commands.core.command.CompositeTransactionalCommand cmd = new org.eclipse.gmf.runtime.emf.commands.core.command.CompositeTransactionalCommand(getEditingDomain(), null);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("cmd.setTransactionNestingEnabled(true);");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("org.eclipse.emf.ecore.EAnnotation annotation = view.getEAnnotation(\"Shortcut\");");
    CharSequence _nonNLS = this._common.nonNLS();
    _builder.append(_nonNLS, "\t\t\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("if (annotation == null) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("// there are indirectly referenced children, need extra commands: ");
    final Function1<GenCompartment, EList<GenChildNode>> _function = (GenCompartment c) -> {
      return c.getChildNodes();
    };
    final Function1<GenChildNode, Boolean> _function_1 = (GenChildNode gcn) -> {
      boolean _isDirectlyOwned = this._utils_qvto.isDirectlyOwned(gcn, it);
      return Boolean.valueOf((!_isDirectlyOwned));
    };
    boolean _exists = IterableExtensions.<GenChildNode>exists(NodeItemSemanticEditPolicy.<GenChildNode>union(it.getChildNodes(), Iterables.<GenChildNode>concat(ListExtensions.<GenCompartment, EList<GenChildNode>>map(it.getCompartments(), _function))), _function_1);
    _builder.append(_exists, "\t\t");
    _builder.newLineIfNotEmpty();
    {
      boolean _hasChildrenOrCompartments = this._utils_qvto.hasChildrenOrCompartments(it);
      if (_hasChildrenOrCompartments) {
        _builder.append("\t\t");
        _builder.append("addDestroyChildNodesCommand(cmd);");
        _builder.newLine();
      }
    }
    _builder.append("\t\t");
    _builder.append("addDestroyShortcutsCommand(cmd, view);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("// delete host element");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("java.util.List<org.eclipse.emf.ecore.EObject> todestroy=new java.util.ArrayList<org.eclipse.emf.ecore.EObject>();");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("todestroy.add(req.getElementToDestroy());");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("//cmd.add(new org.eclipse.gmf.runtime.emf.type.core.commands.DestroyElementCommand(req));");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("cmd.add(new org.eclipse.papyrus.infra.emf.gmf.command.EMFtoGMFCommandWrapper(new org.eclipse.emf.edit.command.DeleteCommand(getEditingDomain(),todestroy )));");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("} else {");
    _builder.append("\t\t\t\tcmd.add(new org.eclipse.gmf.runtime.diagram.core.commands.DeleteCommand(getEditingDomain(), view));");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return getGEFWrapper(cmd.reduce());");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }

  public CharSequence addDestroyChildNodesCommand(final GenNode it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("protected void addDestroyChildNodesCommand(org.eclipse.gmf.runtime.common.core.command.ICompositeCommand cmd) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.gmf.runtime.notation.View view = (org.eclipse.gmf.runtime.notation.View) getHost().getModel();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (java.util.Iterator<?> nit = view.getChildren().iterator(); nit.hasNext();) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.gmf.runtime.notation.Node node = (org.eclipse.gmf.runtime.notation.Node) nit.next();");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("String vid = ");
    CharSequence _visualIDMethodCall = this.xptVisualIDRegistry.getVisualIDMethodCall(it.getDiagram());
    _builder.append(_visualIDMethodCall, "\t\t");
    _builder.append("(node);");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("if (vid != null) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("switch (vid) {");
    _builder.newLine();
    {
      EList<GenChildNode> _childNodes = it.getChildNodes();
      for(final GenChildNode cn : _childNodes) {
        _builder.append("\t\t\t");
        CharSequence _destroyChildNodes = this.destroyChildNodes(cn, "node", it);
        _builder.append(_destroyChildNodes, "\t\t\t");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<GenCompartment> _compartments = it.getCompartments();
      for(final GenCompartment compartment : _compartments) {
        _builder.append("\t\t\t");
        CharSequence _caseVisualID = this.xptVisualIDRegistry.caseVisualID(compartment);
        _builder.append(_caseVisualID, "\t\t\t");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t\t");
        _builder.append("\t");
        _builder.append("for (java.util.Iterator<?> cit = node.getChildren().iterator(); cit.hasNext();) {");
        _builder.newLine();
        _builder.append("\t\t\t");
        _builder.append("\t\t");
        _builder.append("org.eclipse.gmf.runtime.notation.Node cnode = (org.eclipse.gmf.runtime.notation.Node) cit.next();");
        _builder.newLine();
        _builder.append("\t\t\t");
        _builder.append("\t\t");
        _builder.append("String cvid = ");
        CharSequence _visualIDMethodCall_1 = this.xptVisualIDRegistry.getVisualIDMethodCall(it.getDiagram());
        _builder.append(_visualIDMethodCall_1, "\t\t\t\t\t");
        _builder.append("(cnode);");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t\t");
        _builder.append("\t\t");
        _builder.append("if (cvid != null) {");
        _builder.newLine();
        _builder.append("\t\t\t");
        _builder.append("\t\t\t");
        _builder.append("switch (cvid) {");
        _builder.newLine();
        {
          EList<GenChildNode> _childNodes_1 = compartment.getChildNodes();
          for(final GenChildNode cn_1 : _childNodes_1) {
            _builder.append("\t\t\t");
            _builder.append("\t\t\t");
            CharSequence _destroyChildNodes_1 = this.destroyChildNodes(cn_1, "cnode", it);
            _builder.append(_destroyChildNodes_1, "\t\t\t\t\t\t");
            _builder.newLineIfNotEmpty();
          }
        }
        _builder.append("\t\t\t");
        _builder.append("\t\t\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t\t\t");
        _builder.append("\t\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t\t\t");
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t\t\t");
        _builder.append("\t");
        _builder.append("break;");
        _builder.newLine();
      }
    }
    _builder.append("\t\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }

  public CharSequence destroyChildNodes(final GenChildNode it, final String nodeVar, final GenNode genNode) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _caseVisualID = this.xptVisualIDRegistry.caseVisualID(it);
    _builder.append(_caseVisualID);
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _destroyEdges = this.destroyEdges(it, nodeVar);
    _builder.append(_destroyEdges, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("cmd.add(new org.eclipse.gmf.runtime.emf.type.core.commands.DestroyElementCommand(new org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest(getEditingDomain(), ");
    _builder.append(nodeVar, "\t");
    _builder.append(".getElement(), false))); // directlyOwned: ");
    boolean _isDirectlyOwned = this._utils_qvto.isDirectlyOwned(it, genNode);
    _builder.append(_isDirectlyOwned, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("// don\'t need explicit deletion of ");
    _builder.append(nodeVar, "\t");
    _builder.append(" as parent\'s view deletion would clean child views as well");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("// cmd.add(new org.eclipse.gmf.runtime.diagram.core.commands.DeleteCommand(getEditingDomain(), ");
    _builder.append(nodeVar, "\t");
    _builder.append("));");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("break;");
    _builder.newLine();
    return _builder;
  }

  /**
   * @param view - Notation element for the passed node
   * assumes 'cmd' to point to composite command
   */
  public CharSequence destroyEdges(final GenNode it, final String view) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _isEmpty = it.getGenIncomingLinks().isEmpty();
      boolean _not = (!_isEmpty);
      if (_not) {
        _builder.append("for (java.util.Iterator<?> it = ");
        _builder.append(view);
        _builder.append(".getTargetEdges().iterator(); it.hasNext();) {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("org.eclipse.gmf.runtime.notation.Edge incomingLink = (org.eclipse.gmf.runtime.notation.Edge) it.next();");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("String vid = ");
        CharSequence _visualIDMethodCall = this.xptVisualIDRegistry.getVisualIDMethodCall(it.getDiagram());
        _builder.append(_visualIDMethodCall, "\t");
        _builder.append("(incomingLink);");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("if (vid != null) {");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("switch(vid) {");
        _builder.newLine();
        {
          final Function1<GenLink, Boolean> _function = (GenLink l) -> {
            LinkModelFacet _modelFacet = l.getModelFacet();
            return Boolean.valueOf((_modelFacet instanceof FeatureLinkModelFacet));
          };
          boolean _isEmpty_1 = IterableExtensions.isEmpty(IterableExtensions.<GenLink>filter(it.getGenIncomingLinks(), _function));
          boolean _not_1 = (!_isEmpty_1);
          if (_not_1) {
            {
              final Function1<GenLink, Boolean> _function_1 = (GenLink l) -> {
                LinkModelFacet _modelFacet = l.getModelFacet();
                return Boolean.valueOf((_modelFacet instanceof FeatureLinkModelFacet));
              };
              Iterable<GenLink> _filter = IterableExtensions.<GenLink>filter(it.getGenIncomingLinks(), _function_1);
              for(final GenLink il : _filter) {
                _builder.append("\t\t\t");
                _builder.append("case ");
                CharSequence _visualID = VisualIDRegistry.visualID(il);
                _builder.append(_visualID, "\t\t\t");
                _builder.append(":");
                _builder.newLineIfNotEmpty();
              }
            }
            _builder.append("\t\t\t");
            _builder.append("org.eclipse.gmf.runtime.emf.type.core.requests.DestroyReferenceRequest destroyRefReq = new org.eclipse.gmf.runtime.emf.type.core.requests.DestroyReferenceRequest(incomingLink.getSource().getElement(), null, incomingLink.getTarget().getElement(), false);");
            _builder.newLine();
            _builder.append("\t\t\t");
            _builder.append("cmd.add(new org.eclipse.gmf.runtime.emf.type.core.commands.DestroyReferenceCommand(destroyRefReq));");
            _builder.newLine();
            _builder.append("\t\t\t");
            _builder.append("cmd.add(new org.eclipse.gmf.runtime.diagram.core.commands.DeleteCommand(getEditingDomain(), incomingLink));");
            _builder.newLine();
            _builder.append("\t\t\t");
            _builder.append("break;");
            _builder.newLine();
          }
        }
        {
          final Function1<GenLink, Boolean> _function_2 = (GenLink l) -> {
            LinkModelFacet _modelFacet = l.getModelFacet();
            return Boolean.valueOf((_modelFacet instanceof TypeLinkModelFacet));
          };
          boolean _isEmpty_2 = IterableExtensions.isEmpty(IterableExtensions.<GenLink>filter(it.getGenIncomingLinks(), _function_2));
          boolean _not_2 = (!_isEmpty_2);
          if (_not_2) {
            {
              final Function1<GenLink, Boolean> _function_3 = (GenLink l) -> {
                LinkModelFacet _modelFacet = l.getModelFacet();
                return Boolean.valueOf((_modelFacet instanceof TypeLinkModelFacet));
              };
              Iterable<GenLink> _filter_1 = IterableExtensions.<GenLink>filter(it.getGenIncomingLinks(), _function_3);
              for(final GenLink il_1 : _filter_1) {
                _builder.append("\t\t\t");
                _builder.append("case ");
                CharSequence _visualID_1 = VisualIDRegistry.visualID(il_1);
                _builder.append(_visualID_1, "\t\t\t");
                _builder.append(":");
                _builder.newLineIfNotEmpty();
              }
            }
            _builder.append("\t\t\t");
            _builder.append("org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest destroyEltReq = new org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest(incomingLink.getElement(), false);");
            _builder.newLine();
            _builder.append("\t\t\t");
            _builder.append("cmd.add(new org.eclipse.gmf.runtime.emf.type.core.commands.DestroyElementCommand(destroyEltReq));");
            _builder.newLine();
            _builder.append("\t\t\t");
            _builder.append("cmd.add(new org.eclipse.gmf.runtime.diagram.core.commands.DeleteCommand(getEditingDomain(), incomingLink));");
            _builder.newLine();
            _builder.append("\t\t\t");
            _builder.append("break;");
            _builder.newLine();
          }
        }
        _builder.append("\t\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("}");
        _builder.newLine();
      }
    }
    _builder.newLine();
    {
      boolean _isEmpty_3 = it.getGenOutgoingLinks().isEmpty();
      if (_isEmpty_3) {
        _builder.append("for (java.util.Iterator<?> it = ");
        _builder.append(view);
        _builder.append(".getSourceEdges().iterator(); it.hasNext();) {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("org.eclipse.gmf.runtime.notation.Edge outgoingLink = (org.eclipse.gmf.runtime.notation.Edge) it.next();");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("String vid = ");
        CharSequence _visualIDMethodCall_1 = this.xptVisualIDRegistry.getVisualIDMethodCall(it.getDiagram());
        _builder.append(_visualIDMethodCall_1, "\t");
        _builder.append("(outgoingLink);");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("if (vid != null) {");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("switch(vid) {");
        _builder.newLine();
        {
          final Function1<GenLink, Boolean> _function_4 = (GenLink l) -> {
            LinkModelFacet _modelFacet = l.getModelFacet();
            return Boolean.valueOf((_modelFacet instanceof FeatureLinkModelFacet));
          };
          boolean _isEmpty_4 = IterableExtensions.isEmpty(IterableExtensions.<GenLink>filter(it.getGenOutgoingLinks(), _function_4));
          boolean _not_3 = (!_isEmpty_4);
          if (_not_3) {
            {
              final Function1<GenLink, Boolean> _function_5 = (GenLink l) -> {
                LinkModelFacet _modelFacet = l.getModelFacet();
                return Boolean.valueOf((_modelFacet instanceof FeatureLinkModelFacet));
              };
              Iterable<GenLink> _filter_2 = IterableExtensions.<GenLink>filter(it.getGenOutgoingLinks(), _function_5);
              for(final GenLink ol : _filter_2) {
                _builder.append("\t\t\t");
                _builder.append("case ");
                CharSequence _visualID_2 = VisualIDRegistry.visualID(ol);
                _builder.append(_visualID_2, "\t\t\t");
                _builder.append(":");
                _builder.newLineIfNotEmpty();
              }
            }
            _builder.append("\t\t\t");
            _builder.append("org.eclipse.gmf.runtime.emf.type.core.requests.DestroyReferenceRequest destroyRefReq = new org.eclipse.gmf.runtime.emf.type.core.requests.DestroyReferenceRequest(outgoingLink.getSource().getElement(), null, outgoingLink.getTarget().getElement(), false);");
            _builder.newLine();
            _builder.append("\t\t\t");
            _builder.append("cmd.add(new org.eclipse.gmf.runtime.emf.type.core.commands.DestroyReferenceCommand(destroyRefReq));");
            _builder.newLine();
            _builder.append("\t\t\t");
            _builder.append("cmd.add(new org.eclipse.gmf.runtime.diagram.core.commands.DeleteCommand(getEditingDomain(), outgoingLink));");
            _builder.newLine();
            _builder.append("\t\t\t");
            _builder.append("break;");
            _builder.newLine();
          }
        }
        {
          final Function1<GenLink, Boolean> _function_6 = (GenLink l) -> {
            LinkModelFacet _modelFacet = l.getModelFacet();
            return Boolean.valueOf((_modelFacet instanceof TypeLinkModelFacet));
          };
          boolean _isEmpty_5 = IterableExtensions.isEmpty(IterableExtensions.<GenLink>filter(it.getGenOutgoingLinks(), _function_6));
          boolean _not_4 = (!_isEmpty_5);
          if (_not_4) {
            {
              final Function1<GenLink, Boolean> _function_7 = (GenLink l) -> {
                LinkModelFacet _modelFacet = l.getModelFacet();
                return Boolean.valueOf((_modelFacet instanceof TypeLinkModelFacet));
              };
              Iterable<GenLink> _filter_3 = IterableExtensions.<GenLink>filter(it.getGenOutgoingLinks(), _function_7);
              for(final GenLink ol_1 : _filter_3) {
                _builder.append("\t\t\t");
                _builder.append("case ");
                CharSequence _visualID_3 = VisualIDRegistry.visualID(ol_1);
                _builder.append(_visualID_3, "\t\t\t");
                _builder.append(":");
                _builder.newLineIfNotEmpty();
              }
            }
            _builder.append("\t\t\t");
            _builder.append("org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest destroyEltReq = new org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest(outgoingLink.getElement(), false);");
            _builder.newLine();
            _builder.append("\t\t\t");
            _builder.append("cmd.add(new org.eclipse.gmf.runtime.emf.type.core.commands.DestroyElementCommand(destroyEltReq));");
            _builder.newLine();
            _builder.append("\t\t\t");
            _builder.append("cmd.add(new org.eclipse.gmf.runtime.diagram.core.commands.DeleteCommand(getEditingDomain(), outgoingLink));");
            _builder.newLine();
            _builder.append("\t\t\t");
            _builder.append("break;");
            _builder.newLine();
          }
        }
        _builder.append("\t\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("}");
        _builder.newLine();
      }
    }
    return _builder;
  }

  private static <T extends Object> Iterable<T> union(final Iterable<? extends T> listA, final Iterable<? extends T> listB) {
    List<T> result = CollectionLiterals.<T>newLinkedList();
    Iterables.<T>addAll(result, listA);
    Iterables.<T>addAll(result, listB);
    return result;
  }
}
